#![cfg(target_arch = "x86_64")]
#![allow(dead_code)]

use x86_64::structures::gdt::{Descriptor, GlobalDescriptorTable};
use x86_64::registers::segmentation::{CS, SS, Segment, SegmentSelector};
use x86_64::PrivilegeLevel;
use spin::Mutex;

static mut GDT: GlobalDescriptorTable = GlobalDescriptorTable::new();
static GDT_LOCK: Mutex<u64> = Mutex::new(0); // we cannot lock the GDT directly as it must stay in the same place in memory and that behaves weirdly with mutexes

fn add_entry(descriptor: Descriptor)
{
    GDT_LOCK.lock();
    unsafe { GDT.add_entry(descriptor); }
}

pub fn init()
{
    add_entry(Descriptor::kernel_code_segment());
    add_entry(Descriptor::kernel_data_segment());
    add_entry(Descriptor::user_code_segment());
    add_entry(Descriptor::user_data_segment());
    // TODO: add a TSS.
}

pub fn load_and_reload_segment_registers()
{
    GDT_LOCK.lock();
    unsafe { 
        GDT.load();
        CS::set_reg(SegmentSelector::new(1, PrivilegeLevel::Ring0));
        SS::set_reg(SegmentSelector::new(2, PrivilegeLevel::Ring0));
    }
}